"""
scoping_band_pass
=================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "filter" category
"""

class scoping_band_pass(Operator):
    """The band pass filter returns all the values strictly superior to the min threshold value and stricly inferior to the max threshold value in input.

      available inputs:
        - field (Field, FieldsContainer)
        - min_threshold (float, Field)
        - max_threshold (float, Field)

      available outputs:
        - scoping (Scoping)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.filter.scoping_band_pass()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_min_threshold = float()
      >>> op.inputs.min_threshold.connect(my_min_threshold)
      >>> my_max_threshold = float()
      >>> op.inputs.max_threshold.connect(my_max_threshold)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.filter.scoping_band_pass(field=my_field,min_threshold=my_min_threshold,max_threshold=my_max_threshold)

      >>> # Get output data
      >>> result_scoping = op.outputs.scoping()"""
    def __init__(self, field=None, min_threshold=None, max_threshold=None, config=None, server=None):
        super().__init__(name="core::scoping::band_pass", config = config, server = server)
        self._inputs = InputsScopingBandPass(self)
        self._outputs = OutputsScopingBandPass(self)
        if field !=None:
            self.inputs.field.connect(field)
        if min_threshold !=None:
            self.inputs.min_threshold.connect(min_threshold)
        if max_threshold !=None:
            self.inputs.max_threshold.connect(max_threshold)

    @staticmethod
    def _spec():
        spec = Specification(description="""The band pass filter returns all the values strictly superior to the min threshold value and stricly inferior to the max threshold value in input.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field","fields_container"], optional=False, document="""field or fields container with only one field is expected"""), 
                                 1 : PinSpecification(name = "min_threshold", type_names=["double","field"], optional=False, document="""a min threshold scalar or a field containing one value is expected"""), 
                                 2 : PinSpecification(name = "max_threshold", type_names=["double","field"], optional=False, document="""a max threshold scalar or a field containing one value is expected""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "scoping", type_names=["scoping"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "core::scoping::band_pass")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsScopingBandPass 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsScopingBandPass 
        """
        return super().outputs


#internal name: core::scoping::band_pass
#scripting name: scoping_band_pass
class InputsScopingBandPass(_Inputs):
    """Intermediate class used to connect user inputs to scoping_band_pass operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.filter.scoping_band_pass()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_min_threshold = float()
      >>> op.inputs.min_threshold.connect(my_min_threshold)
      >>> my_max_threshold = float()
      >>> op.inputs.max_threshold.connect(my_max_threshold)
    """
    def __init__(self, op: Operator):
        super().__init__(scoping_band_pass._spec().inputs, op)
        self._field = Input(scoping_band_pass._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)
        self._min_threshold = Input(scoping_band_pass._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._min_threshold)
        self._max_threshold = Input(scoping_band_pass._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._max_threshold)

    @property
    def field(self):
        """Allows to connect field input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_field : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.filter.scoping_band_pass()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

    @property
    def min_threshold(self):
        """Allows to connect min_threshold input to the operator

        - pindoc: a min threshold scalar or a field containing one value is expected

        Parameters
        ----------
        my_min_threshold : float, Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.filter.scoping_band_pass()
        >>> op.inputs.min_threshold.connect(my_min_threshold)
        >>> #or
        >>> op.inputs.min_threshold(my_min_threshold)

        """
        return self._min_threshold

    @property
    def max_threshold(self):
        """Allows to connect max_threshold input to the operator

        - pindoc: a max threshold scalar or a field containing one value is expected

        Parameters
        ----------
        my_max_threshold : float, Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.filter.scoping_band_pass()
        >>> op.inputs.max_threshold.connect(my_max_threshold)
        >>> #or
        >>> op.inputs.max_threshold(my_max_threshold)

        """
        return self._max_threshold

class OutputsScopingBandPass(_Outputs):
    """Intermediate class used to get outputs from scoping_band_pass operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.filter.scoping_band_pass()
      >>> # Connect inputs : op.inputs. ...
      >>> result_scoping = op.outputs.scoping()
    """
    def __init__(self, op: Operator):
        super().__init__(scoping_band_pass._spec().outputs, op)
        self._scoping = Output(scoping_band_pass._spec().output_pin(0), 0, op) 
        self._outputs.append(self._scoping)

    @property
    def scoping(self):
        """Allows to get scoping output of the operator


        Returns
        ----------
        my_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.filter.scoping_band_pass()
        >>> # Connect inputs : op.inputs. ...
        >>> result_scoping = op.outputs.scoping() 
        """
        return self._scoping




"""
fft_eval
========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Math plugin, from "math" category
"""

class fft_eval(Operator):
    """Evaluate the fast fourier transforms at a given set of fields.

      available inputs:
        - field_t (Field)
        - time_scoping (Scoping) (optional)

      available outputs:
        - field (Field)
        - offset (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.math.fft_eval()

      >>> # Make input connections
      >>> my_field_t = dpf.Field()
      >>> op.inputs.field_t.connect(my_field_t)
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.math.fft_eval(field_t=my_field_t,time_scoping=my_time_scoping)

      >>> # Get output data
      >>> result_field = op.outputs.field()
      >>> result_offset = op.outputs.offset()"""
    def __init__(self, field_t=None, time_scoping=None, config=None, server=None):
        super().__init__(name="fft_eval", config = config, server = server)
        self._inputs = InputsFftEval(self)
        self._outputs = OutputsFftEval(self)
        if field_t !=None:
            self.inputs.field_t.connect(field_t)
        if time_scoping !=None:
            self.inputs.time_scoping.connect(time_scoping)

    @staticmethod
    def _spec():
        spec = Specification(description="""Evaluate the fast fourier transforms at a given set of fields.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field_t", type_names=["field"], optional=False, document="""field of values to evaluate"""), 
                                 1 : PinSpecification(name = "time_scoping", type_names=["scoping"], optional=True, document="""if specified only the results at these set ids are used""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "offset", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "fft_eval")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFftEval 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsFftEval 
        """
        return super().outputs


#internal name: fft_eval
#scripting name: fft_eval
class InputsFftEval(_Inputs):
    """Intermediate class used to connect user inputs to fft_eval operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.fft_eval()
      >>> my_field_t = dpf.Field()
      >>> op.inputs.field_t.connect(my_field_t)
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
    """
    def __init__(self, op: Operator):
        super().__init__(fft_eval._spec().inputs, op)
        self._field_t = Input(fft_eval._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field_t)
        self._time_scoping = Input(fft_eval._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._time_scoping)

    @property
    def field_t(self):
        """Allows to connect field_t input to the operator

        - pindoc: field of values to evaluate

        Parameters
        ----------
        my_field_t : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_eval()
        >>> op.inputs.field_t.connect(my_field_t)
        >>> #or
        >>> op.inputs.field_t(my_field_t)

        """
        return self._field_t

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator

        - pindoc: if specified only the results at these set ids are used

        Parameters
        ----------
        my_time_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_eval()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> #or
        >>> op.inputs.time_scoping(my_time_scoping)

        """
        return self._time_scoping

class OutputsFftEval(_Outputs):
    """Intermediate class used to get outputs from fft_eval operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.fft_eval()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
      >>> result_offset = op.outputs.offset()
    """
    def __init__(self, op: Operator):
        super().__init__(fft_eval._spec().outputs, op)
        self._field = Output(fft_eval._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)
        self._offset = Output(fft_eval._spec().output_pin(2), 2, op) 
        self._outputs.append(self._offset)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_eval()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field

    @property
    def offset(self):
        """Allows to get offset output of the operator


        Returns
        ----------
        my_offset : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_eval()
        >>> # Connect inputs : op.inputs. ...
        >>> result_offset = op.outputs.offset() 
        """
        return self._offset




"""
unit_convert
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "math" category
"""

class unit_convert(Operator):
    """Convert an input field/fields container or mesh of a given unit to another unit.

      available inputs:
        - entity_to_convert (Field, FieldsContainer, MeshedRegion, MeshesContainer)
        - unit_name (str)

      available outputs:
        - converted_entity (Field ,FieldsContainer ,MeshedRegion ,MeshesContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.math.unit_convert()

      >>> # Make input connections
      >>> my_entity_to_convert = dpf.Field()
      >>> op.inputs.entity_to_convert.connect(my_entity_to_convert)
      >>> my_unit_name = str()
      >>> op.inputs.unit_name.connect(my_unit_name)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.math.unit_convert(entity_to_convert=my_entity_to_convert,unit_name=my_unit_name)

      >>> # Get output data
      >>> result_converted_entity = op.outputs.converted_entity()"""
    def __init__(self, entity_to_convert=None, unit_name=None, config=None, server=None):
        super().__init__(name="unit_convert", config = config, server = server)
        self._inputs = InputsUnitConvert(self)
        self._outputs = OutputsUnitConvert(self)
        if entity_to_convert !=None:
            self.inputs.entity_to_convert.connect(entity_to_convert)
        if unit_name !=None:
            self.inputs.unit_name.connect(unit_name)

    @staticmethod
    def _spec():
        spec = Specification(description="""Convert an input field/fields container or mesh of a given unit to another unit.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "entity_to_convert", type_names=["field","fields_container","abstract_meshed_region","meshes_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "unit_name", type_names=["string"], optional=False, document="""unit as a string, ex 'm' for meter, 'Pa' for pascal,...""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "converted_entity", type_names=["field","fields_container","abstract_meshed_region","meshes_container"], optional=False, document="""the output entity is the same as the input (inplace operator)""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "unit_convert")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsUnitConvert 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsUnitConvert 
        """
        return super().outputs


#internal name: unit_convert
#scripting name: unit_convert
class InputsUnitConvert(_Inputs):
    """Intermediate class used to connect user inputs to unit_convert operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.unit_convert()
      >>> my_entity_to_convert = dpf.Field()
      >>> op.inputs.entity_to_convert.connect(my_entity_to_convert)
      >>> my_unit_name = str()
      >>> op.inputs.unit_name.connect(my_unit_name)
    """
    def __init__(self, op: Operator):
        super().__init__(unit_convert._spec().inputs, op)
        self._entity_to_convert = Input(unit_convert._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._entity_to_convert)
        self._unit_name = Input(unit_convert._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._unit_name)

    @property
    def entity_to_convert(self):
        """Allows to connect entity_to_convert input to the operator

        Parameters
        ----------
        my_entity_to_convert : Field, FieldsContainer, MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.unit_convert()
        >>> op.inputs.entity_to_convert.connect(my_entity_to_convert)
        >>> #or
        >>> op.inputs.entity_to_convert(my_entity_to_convert)

        """
        return self._entity_to_convert

    @property
    def unit_name(self):
        """Allows to connect unit_name input to the operator

        - pindoc: unit as a string, ex 'm' for meter, 'Pa' for pascal,...

        Parameters
        ----------
        my_unit_name : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.unit_convert()
        >>> op.inputs.unit_name.connect(my_unit_name)
        >>> #or
        >>> op.inputs.unit_name(my_unit_name)

        """
        return self._unit_name

class OutputsUnitConvert(_Outputs):
    """Intermediate class used to get outputs from unit_convert operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.unit_convert()
      >>> # Connect inputs : op.inputs. ...
      >>> result_converted_entity = op.outputs.converted_entity()
    """
    def __init__(self, op: Operator):
        super().__init__(unit_convert._spec().outputs, op)
        self.converted_entity_as_field = Output( _modify_output_spec_with_one_type(unit_convert._spec().output_pin(0), "field"), 0, op) 
        self._outputs.append(self.converted_entity_as_field)
        self.converted_entity_as_fields_container = Output( _modify_output_spec_with_one_type(unit_convert._spec().output_pin(0), "fields_container"), 0, op) 
        self._outputs.append(self.converted_entity_as_fields_container)
        self.converted_entity_as_meshed_region = Output( _modify_output_spec_with_one_type(unit_convert._spec().output_pin(0), "abstract_meshed_region"), 0, op) 
        self._outputs.append(self.converted_entity_as_meshed_region)
        self.converted_entity_as_meshes_container = Output( _modify_output_spec_with_one_type(unit_convert._spec().output_pin(0), "meshes_container"), 0, op) 
        self._outputs.append(self.converted_entity_as_meshes_container)




"""
elemental_to_nodal_fc
=====================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "averaging" category
"""

class elemental_to_nodal_fc(Operator):
    """Transform ElementalNodal fields to Nodal fields, compute result on a given node scoping.

      available inputs:
        - fields_container (FieldsContainer)
        - mesh (MeshedRegion, MeshesContainer) (optional)
        - force_averaging (int) (optional)
        - mesh_scoping (Scoping, ScopingsContainer) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.averaging.elemental_to_nodal_fc()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_force_averaging = int()
      >>> op.inputs.force_averaging.connect(my_force_averaging)
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.averaging.elemental_to_nodal_fc(fields_container=my_fields_container,mesh=my_mesh,force_averaging=my_force_averaging,mesh_scoping=my_mesh_scoping)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, fields_container=None, mesh=None, force_averaging=None, mesh_scoping=None, config=None, server=None):
        super().__init__(name="elemental_to_nodal_fc", config = config, server = server)
        self._inputs = InputsElementalToNodalFc(self)
        self._outputs = OutputsElementalToNodalFc(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if force_averaging !=None:
            self.inputs.force_averaging.connect(force_averaging)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)

    @staticmethod
    def _spec():
        spec = Specification(description="""Transform ElementalNodal fields to Nodal fields, compute result on a given node scoping.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region","meshes_container"], optional=True, document=""""""), 
                                 2 : PinSpecification(name = "force_averaging", type_names=["int32"], optional=True, document="""averaging on nodes is used if this pin is set to 1 (default is one for integrated results and 0 for dicrete ones)"""), 
                                 3 : PinSpecification(name = "mesh_scoping", type_names=["scoping","scopings_container"], optional=True, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "elemental_to_nodal_fc")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsElementalToNodalFc 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsElementalToNodalFc 
        """
        return super().outputs


#internal name: elemental_to_nodal_fc
#scripting name: elemental_to_nodal_fc
class InputsElementalToNodalFc(_Inputs):
    """Intermediate class used to connect user inputs to elemental_to_nodal_fc operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.elemental_to_nodal_fc()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_force_averaging = int()
      >>> op.inputs.force_averaging.connect(my_force_averaging)
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    """
    def __init__(self, op: Operator):
        super().__init__(elemental_to_nodal_fc._spec().inputs, op)
        self._fields_container = Input(elemental_to_nodal_fc._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._mesh = Input(elemental_to_nodal_fc._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh)
        self._force_averaging = Input(elemental_to_nodal_fc._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._force_averaging)
        self._mesh_scoping = Input(elemental_to_nodal_fc._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._mesh_scoping)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_to_nodal_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        Parameters
        ----------
        my_mesh : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_to_nodal_fc()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def force_averaging(self):
        """Allows to connect force_averaging input to the operator

        - pindoc: averaging on nodes is used if this pin is set to 1 (default is one for integrated results and 0 for dicrete ones)

        Parameters
        ----------
        my_force_averaging : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_to_nodal_fc()
        >>> op.inputs.force_averaging.connect(my_force_averaging)
        >>> #or
        >>> op.inputs.force_averaging(my_force_averaging)

        """
        return self._force_averaging

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        Parameters
        ----------
        my_mesh_scoping : Scoping, ScopingsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_to_nodal_fc()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

class OutputsElementalToNodalFc(_Outputs):
    """Intermediate class used to get outputs from elemental_to_nodal_fc operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.elemental_to_nodal_fc()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(elemental_to_nodal_fc._spec().outputs, op)
        self._fields_container = Output(elemental_to_nodal_fc._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_to_nodal_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




from ansys.grpc.dpf import field_pb2, base_pb2, field_pb2_grpc
from ansys.dpf.core import scoping
from ansys.dpf.core.common import natures, locations
from ansys.dpf.core import errors
from ansys.dpf.core import server as serverlib

import numpy as np


class _FieldBase:
    """Contains base APIs for all implementations that follow DPF's field concept."""

    def __init__(
        self,
        nentities=0,
        nature=natures.vector,
        location=locations.nodal,
        is_property_field=False,
        field=None,
        server=None,
    ):
        """Initialize the field either with an optional field message or by connecting to a stub."""
        if server is None:
            server = serverlib._global_server()

        self._server = server
        self._stub = self._connect()

        if field is None:
            request = field_pb2.FieldRequest()
            if hasattr(nature, "name"):
                snature = nature.name
            else:
                snature = nature
            request.nature = base_pb2.Nature.Value(snature.upper())
            request.location.location = location
            request.size.scoping_size = nentities
            if snature == natures.vector.name:
                elem_data_size = 3
            elif snature == natures.symmatrix.name:
                elem_data_size = 6
            else:
                elem_data_size = 1
            request.size.data_size = nentities * elem_data_size
            if is_property_field:
                request.datatype = "int"
            self._message = self._stub.Create(request)
        else:
            from ansys.dpf.core import field as field_module
            from ansys.dpf.core import property_field

            if isinstance(field, field_module.Field):
                self._message = field._message
            elif isinstance(field, property_field.PropertyField):
                self._message = field._message
            elif isinstance(field, field_pb2.Field):
                self._message = field
            else:
                raise TypeError(f'Cannot create a field from a "{type(field)}" object')

    @property
    def shape(self):
        """Numpy-like shape of the field.

        Examples
        --------
        tuple
            Shape of a stress field.

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_transient_result())
        >>> s_op =model.results.stress()
        >>> s_fc = s_op.outputs.fields_container()
        >>> field = s_fc[0]
        >>> field.shape
        (5720, 6)

        """
        if self.component_count != 1:
            return (self.elementary_data_count, self.component_count)
        return self.elementary_data_count

    @property
    def component_count(self):
        """Number of components in each elementary data of the field.

        Returns
        -------
        int
            Number of components in each elementary data of the field.
        """
        request = field_pb2.CountRequest()
        request.entity = base_pb2.NUM_COMPONENT
        request.field.CopyFrom(self._message)
        return self._stub.Count(request).count

    @property
    def elementary_data_count(self):
        """Number of elementary data in the field.

        Returns
        -------
        int
            Number of elementary data in the field.

        """
        request = field_pb2.CountRequest()
        request.entity = base_pb2.NUM_ELEMENTARY_DATA
        request.field.CopyFrom(self._message)
        return self._stub.Count(request).count

    @property
    def size(self):
        """Length of the data vector.

        The length is equal to the number of elementary data times the number of components.

        Returns
        -------
        int
            Length of the data vector.

        """
        return self.elementary_data_count * self.component_count

    @property
    def elementary_data_shape(self):
        """Numpy-like shape of the field."""
        if self.component_count != 1:
            return (1, self.component_count)
        else:
            return self.component_count

    @property
    def ndim(self):
        return self.component_count

    def __str__(self):
        """Describes the entity.

        Returns
        -------
        str
            Description of the entity.

        """
        from ansys.dpf.core.core import _description

        return _description(self._message, self._server)

    def __len__(self):
        return self.size

    def _del_scoping(self, scope):
        scope.__del__()

    def __del__(self):
        try:
            self._stub.Delete(self._message)
        except:
            pass

    def _connect(self):
        """Connect to the gRPC service."""
        return field_pb2_grpc.FieldServiceStub(self._server.channel)

    def _set_scoping(self, scoping):
        """Set the scoping.

        Parameters
        ----------
        scoping : :class:`ansys.dpf.core.scoping.Scoping`

        """
        request = field_pb2.UpdateScopingRequest()
        request.scoping.CopyFrom(scoping._message)
        request.field.CopyFrom(self._message)
        self._stub.UpdateScoping(request)

    def _get_scoping(self):
        """Retrieve the scoping.

        Returns
        -------
        scoping : :class:`ansys.dpf.core.scoping.Scoping`

        """
        request = field_pb2.GetRequest()
        request.field.CopyFrom(self._message)
        message = self._stub.GetScoping(request)
        return scoping.Scoping(scoping=message.scoping, server=self._server)

    @property
    def scoping(self):
        """Scoping specifying where the data is.

        Each entity data is on a given scoping ID.

        Returns
        -------
        scoping : :class:`ansys.dpf.core.scoping.Scoping`

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> stress_op = model.results.stress()
        >>> fields_container = stress_op.outputs.fields_container()
        >>> scoping = fields_container[0].scoping
        >>> scoping.location
        'Elemental'
        >>> scoping.id(3)
        586
        >>> #The fourth elementary data of the field corresponds to
        >>> #the element id number 586 in the mesh
        """

        return self._get_scoping()

    @scoping.setter
    def scoping(self, scoping):
        return self._set_scoping(scoping)

    def get_entity_data(self, index):
        """Retrieves the elementary data of the scoping's index in an array.

        Returns
        --------
        numpy.ndarray

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> stress_op = model.results.stress()
        >>> fields_container = stress_op.outputs.fields_container()
        >>> fields_container[0].get_entity_data(0)
        array([[-3.27795062e+05,  1.36012200e+06,  1.49090608e+08,
                -4.88688900e+06,  1.43038560e+07,  1.65455040e+07],
               [-4.63817550e+06,  1.29312225e+06,  1.20411832e+08,
                -6.06617800e+06,  2.34829700e+07,  1.77231120e+07],
               [-2.35684860e+07, -3.53474400e+07,  2.01501168e+08,
                -5.23361700e+06, -2.88789280e+07, -6.16478200e+06],
               [-3.92756960e+07, -2.72369280e+07,  1.81454016e+08,
                -3.75441450e+06, -3.62480300e+06, -3.26075620e+07],
               [ 1.63554530e+07,  2.83190520e+07,  1.05084256e+08,
                -1.30219020e+07,  5.19906719e+05,  8.82430200e+06],
               [ 1.80755620e+07,  5.25578750e+06,  7.76211600e+07,
                -7.53063750e+06,  2.44717000e+06,  2.92675125e+06],
               [ 9.25567760e+07,  8.15244320e+07,  2.77157632e+08,
                -1.48489875e+06,  5.89250600e+07,  2.05608920e+07],
               [ 6.70443680e+07,  8.70343440e+07,  2.73050464e+08,
                -2.48670150e+06,  1.52268930e+07,  6.09583280e+07]])

        """
        request = field_pb2.GetElementaryDataRequest()
        request.field.CopyFrom(self._message)
        request.index = index
        list_message = self._stub.GetElementaryData(
            request, metadata=[(b"float_or_double", b"double")]
        )
        data = []
        if list_message.elemdata_containers.data.HasField("datadouble"):
            data = list_message.elemdata_containers.data.datadouble.rep_double
        elif list_message.elemdata_containers.data.HasField("dataint"):
            data = list_message.elemdata_containers.data.dataint.rep_int

        array = np.array(data)
        if self.component_count != 1:
            n_comp = self.component_count
            array = array.reshape((len(data) // n_comp, n_comp))

        return array

    def get_entity_data_by_id(self, id):
        """Retrieve the data of the scoping's ID in the parameter of the field in an array.

        Returns
        -------
        numpy.ndarray
            Data based on the scoping ID.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> stress_op = model.results.stress()
        >>> fields_container = stress_op.outputs.fields_container()
        >>> fields_container[0].get_entity_data_by_id(391)
        array([[-3.27795062e+05,  1.36012200e+06,  1.49090608e+08,
                -4.88688900e+06,  1.43038560e+07,  1.65455040e+07],
               [-4.63817550e+06,  1.29312225e+06,  1.20411832e+08,
                -6.06617800e+06,  2.34829700e+07,  1.77231120e+07],
               [-2.35684860e+07, -3.53474400e+07,  2.01501168e+08,
                -5.23361700e+06, -2.88789280e+07, -6.16478200e+06],
               [-3.92756960e+07, -2.72369280e+07,  1.81454016e+08,
                -3.75441450e+06, -3.62480300e+06, -3.26075620e+07],
               [ 1.63554530e+07,  2.83190520e+07,  1.05084256e+08,
                -1.30219020e+07,  5.19906719e+05,  8.82430200e+06],
               [ 1.80755620e+07,  5.25578750e+06,  7.76211600e+07,
                -7.53063750e+06,  2.44717000e+06,  2.92675125e+06],
               [ 9.25567760e+07,  8.15244320e+07,  2.77157632e+08,
                -1.48489875e+06,  5.89250600e+07,  2.05608920e+07],
               [ 6.70443680e+07,  8.70343440e+07,  2.73050464e+08,
                -2.48670150e+06,  1.52268930e+07,  6.09583280e+07]])

        """
        index = self.scoping.index(id)
        if index < 0:
            raise ValueError(f"The ID {id} must be greater than 0.")
        return self.get_entity_data(index)

    def append(self, data, scopingid):
        """Add an entity data to the existing data.

        Parameters
        ----------
        data : list of int, double, or array
          Data in the entity.
        scopingid : int
            ID of the scoping.

        Examples
        --------
        >>> from ansys.dpf.core import fields_factory
        >>> field = fields_factory.create_3d_vector_field(2)
        >>> field.append([1.,2.,3.],1)
        >>> field.append([1.,2.,3.],2)
        >>> field.data
        array([[1., 2., 3.],
               [1., 2., 3.]])
        >>> field.scoping.ids
        [1, 2]

        """
        if isinstance(data, (np.ndarray, np.generic)):
            data = data.reshape(data.size).tolist()
        elif len(data) > 0 and isinstance(data[0], list):
            data = np.array(data)
            data = data.reshape(data.size).tolist()
        request = field_pb2.AddDataRequest()
        if self._message.datatype == "int":
            request.elemdata_containers.data.dataint.rep_int.extend(data)
        else:
            request.elemdata_containers.data.datadouble.rep_double.extend(data)
        request.elemdata_containers.scoping_id = scopingid

        request.field.CopyFrom(self._message)
        self._stub.AddData(request)

    @property
    def _data_pointer(self):
        """First index of each entity data.

        Returns
        -------
        numpy.ndarray
            Data in the field.

        Notes
        -----
        Print a progress bar.

        """
        request = field_pb2.ListRequest()
        request.field.CopyFrom(self._message)
        service = self._stub.ListDataPointer(request)
        dtype = np.int32
        return scoping._data_get_chunk_(dtype, service)

    @property
    def _data_pointer_as_list(self):
        """First index of each entity data.

        Returns
        -------
        list
            List of first indexes of each data data.

        Notes
        -----
        Print a progress bar.

        """
        request = field_pb2.ListRequest()
        request.field.CopyFrom(self._message)
        service = self._stub.ListDataPointer(request)
        dtype = np.int32
        return scoping._data_get_chunk_(dtype, service, False)

    @_data_pointer.setter
    def _data_pointer(self, data):
        self._set_data_pointer(data)

    def _set_data_pointer(self, data):
        if isinstance(data, (np.ndarray, np.generic)):
            data = np.array(data.reshape(data.size), dtype=np.int32)
        else:
            data = np.array(data, dtype=np.int32)
        if data.size == 0:
            return
        metadata = [("size_int", f"{len(data)}")]
        request = field_pb2.UpdateDataRequest()
        request.field.CopyFrom(self._message)
        self._stub.UpdateDataPointer(
            scoping._data_chunk_yielder(request, data), metadata=metadata
        )

    @property
    def data(self):
        """Data in the field as an array.

        Notes
        -----
        Print a progress bar.

        Returns
        -------
        numpy.ndarray
            Data in the field.
        """
        return self._get_data()

    @property
    def data_as_list(self):
        """Data in the field as a Python list.

        Returns
        -------
        List
            List of the data in the field.

        Notes
        -----
        Print a progress bar.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> disp = model.results.displacement()
        >>> fields_container = disp.outputs.fields_container()
        >>> field = fields_container[0]
        >>> # field.data_as_list

        """
        return self._get_data(np_array=False)

    def _get_data(self, np_array=True):
        request = field_pb2.ListRequest()
        request.field.CopyFrom(self._message)
        if self._message.datatype == "int":
            data_type = "int"
            dtype = np.int32
        else:
            data_type = "double"
            dtype = np.float
        service = self._stub.List(request, metadata=[("float_or_double", data_type)])
        array = scoping._data_get_chunk_(dtype, service, np_array)

        ncomp = self.component_count
        if ncomp != 1 and np_array:
            array = array.reshape(self.shape)

        return array

    @data.setter
    def data(self, data):
        self._set_data(data)

    def _set_data(self, data):
        if self._message.datatype == "int":
            if not isinstance(data[0], int) and not isinstance(data[0], np.int32):
                raise errors.InvalidTypeError("data", "list of int")
            data = np.array(data, dtype=np.int32)
            metadata = [("size_int", f"{len(data)}")]
        else:
            if isinstance(data, (np.ndarray, np.generic)):
                if (
                    0 != self.size
                    and self.component_count > 1
                    and data.size // self.component_count
                    != data.size / self.component_count
                ):
                    raise ValueError(
                        f"An array of shape {self.shape} is expected and "
                        f"shape {data.shape} was input"
                    )
                else:
                    data = np.array(data.reshape(data.size), dtype=float)
            else:
                data = np.array(data, dtype=float)
            metadata = [("float_or_double", "double"), ("size_double", f"{len(data)}")]
        request = field_pb2.UpdateDataRequest()
        request.field.CopyFrom(self._message)
        self._stub.UpdateData(
            scoping._data_chunk_yielder(request, data), metadata=metadata
        )


class _LocalFieldBase(_FieldBase):
    """Caches the internal data of the field so that it can be modified locally.

    A single update request is sent to the server when the local field is deleted.

    Parameters
    ----------
    field : _FieldBase
        Field to copy locally.

    """

    def __init__(self, field):
        self._message = field._message
        self._server = field._server
        self._stub = field._stub
        self._is_property_field = field._message.datatype == "int"
        self._owner_field = field
        self.__cache_data__()

    def __cache_data__(self):
        self._ncomp = super().component_count
        self._data_copy = super().data_as_list
        self._num_entities_reserved = len(self._data_copy)
        self._data_pointer_copy = super()._data_pointer_as_list
        self._scoping_ids_copy = super().scoping.ids
        self._num_entities = len(self._scoping_ids_copy)
        self._has_data_pointer = len(self._data_pointer_copy) > 0

    @property
    def size(self):
        """Length of the data vector.

        Length equals the number of elementary data times the number of components.

        Returns
        -------
        int
            Length of the data vector.

        """
        return len(self._data_copy)

    def get_entity_data(self, index):
        """Retrieve the elementary data of the scoping's index as an array.

        Returns
        -------
        numpy.ndarray

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> stress_op = model.results.stress()
        >>> fields_container = stress_op.outputs.fields_container()
        >>> field = fields_container[0]
        >>> with field.as_local_field() as f:
        ...     print(f.get_entity_data(0))
        [[-3.27795062e+05  1.36012200e+06  1.49090608e+08 -4.88688900e+06
           1.43038560e+07  1.65455040e+07]
         [-4.63817550e+06  1.29312225e+06  1.20411832e+08 -6.06617800e+06
           2.34829700e+07  1.77231120e+07]
         [-2.35684860e+07 -3.53474400e+07  2.01501168e+08 -5.23361700e+06
          -2.88789280e+07 -6.16478200e+06]
         [-3.92756960e+07 -2.72369280e+07  1.81454016e+08 -3.75441450e+06
          -3.62480300e+06 -3.26075620e+07]
         [ 1.63554530e+07  2.83190520e+07  1.05084256e+08 -1.30219020e+07
           5.19906719e+05  8.82430200e+06]
         [ 1.80755620e+07  5.25578750e+06  7.76211600e+07 -7.53063750e+06
           2.44717000e+06  2.92675125e+06]
         [ 9.25567760e+07  8.15244320e+07  2.77157632e+08 -1.48489875e+06
           5.89250600e+07  2.05608920e+07]
         [ 6.70443680e+07  8.70343440e+07  2.73050464e+08 -2.48670150e+06
           1.52268930e+07  6.09583280e+07]]

        """
        if index > self._num_entities:
            raise ValueError(
                f"Requested scoping {index} is greater than the number of "
                f"available indices {len(self._scoping_ids_copy)}"
            )
        if self._has_data_pointer:
            first_index = self._data_pointer_copy[index]
            if index < len(self._data_pointer_copy) - 1:
                last_index = self._data_pointer_copy[index + 1] - 1
            else:
                last_index = len(self._data_copy) - 1
        else:
            first_index = self._ncomp * index
            last_index = self._ncomp * (index + 1) - 1
        if self._is_property_field:
            array = np.array(
                self._data_copy[first_index : last_index + 1], dtype=np.int32
            )
        else:
            array = np.array(self._data_copy[first_index : last_index + 1])

        if self._ncomp > 1:
            return array.reshape((array.size // self._ncomp, self._ncomp))
        else:
            return array

    def get_entity_data_by_id(self, id):
        """Retrieve the data of the scoping's ID in the parameter of the field.

        Returns
        -------
        numpy.ndarray
            Data based on the scoping ID.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.static_rst)
        >>> stress_op = model.results.stress()
        >>> fields_container = stress_op.outputs.fields_container()
        >>> with fields_container[0].as_local_field() as f:
        ...     for id in f.scoping_ids:
        ...         if id < 2:
        ...             print(f.get_entity_data_by_id(id))
        [[-5.83890625e+03 -1.04498969e+05 -5.83890625e+03  2.10637354e+03
          -2.10637354e+03 -1.45397385e+02]
         [ 3.53322632e+03 -1.00388367e+05  3.53322632e+03 -1.66410352e+03
           1.66410352e+03  5.36620178e+01]
         [-1.05799683e+03 -1.00437922e+05  2.14961670e+03  5.90637268e+02
           1.37861340e+03 -1.68223175e+02]
         [ 2.62742480e+03 -9.89340078e+04 -2.02909998e+03 -2.40310791e+03
          -1.84942798e+03  7.16406616e+02]
         [-2.02909998e+03 -9.89340078e+04  2.62742480e+03  1.84942798e+03
           2.40310791e+03  7.16406616e+02]
         [ 2.14961670e+03 -1.00437922e+05 -1.05799683e+03 -1.37861340e+03
          -5.90637268e+02 -1.68223175e+02]
         [-4.94986755e+02 -9.87357891e+04 -4.94986755e+02 -6.93923187e+01
           6.93923187e+01 -1.59779755e+02]
         [ 1.36953296e+03 -9.76330156e+04  1.36953296e+03 -7.69014221e+02
           7.69014221e+02  4.90502930e+02]]

        """
        index = self._scoping_ids_copy.index(id)
        if index < 0:
            raise ValueError(f"The id {id} doesn't exist in the scoping")
        return self.get_entity_data(index)

    def append(self, data, scopingid):
        """Add an entity data to the existing data.

        Parameters
        ----------
        data : list of int, double or array
            Data for the entity.
        scopingid : int
            ID of the scoping.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> num_entities=100
        >>> field_to_local = dpf.fields_factory.create_3d_vector_field(
        ...     num_entities, location=dpf.locations.elemental_nodal
        ... )
        >>> with field_to_local.as_local_field() as f:
        ...     for i in range(1,num_entities+1):
        ...         f.append([[0.1*i,0.2*i, 0.3*i],[0.1*i,0.2*i, 0.3*i]],i)

        """
        if self._is_property_field:
            if isinstance(data[0], np.int64):
                data = np.array(data, dtype=np.int32)
            if not isinstance(data[0], int) and not isinstance(data[0], np.int32):
                raise errors.InvalidTypeError("data", "list of int")
        if (len(data) > 0 and isinstance(data, list)) or isinstance(
            data, (np.ndarray, np.generic)
        ):
            data = np.array(data).flatten().tolist()

        data_size = len(self._data_copy)
        self._scoping_ids_copy.append(scopingid)
        if len(self._data_pointer_copy) > 0:
            self._data_pointer_copy.append(data_size)

        self._data_copy.extend(data)
        self._num_entities += 1
        if self._has_data_pointer == False:
            if isinstance(data, (np.ndarray, np.generic)):
                data_size = data.size
            else:
                data_size = len(data)
            if data_size > self._ncomp:
                self._data_pointer_copy = [
                    i * self._ncomp for i in range(0, self._num_entities)
                ]
                self._has_data_pointer = True

    def data_as_list(self):
        """Retrieve the data in the field as a Python list.

        Returns
        -------
        list
            List of the data in the field.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> disp = model.results.displacement()
        >>> fields_container = disp.outputs.fields_container()
        >>> field = fields_container[0]
        >>> with field.as_local_field() as f:
        ...     my_data_list = f.data_as_list

        """
        return self._data_copy

    @property
    def data(self):
        """Data in the field.

        Returns
        -------
        numpy.ndarray

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> disp = model.results.displacement()
        >>> fields_container = disp.outputs.fields_container()
        >>> field = fields_container[0]
        >>> with field.as_local_field() as f:
        ...     print(f.data)
        [[ 6.25586668e-03 -1.39243136e-02  2.42697211e-05]
         [ 1.79675948e-02 -2.74812825e-02  1.83822050e-05]
         [-6.72664571e-03 -3.21373459e-02  1.67159110e-04]
         ...
         [-6.07730368e-03  3.22569017e-02  3.10184480e-04]
         [-3.51074714e-06  2.16872928e-08  6.40738989e-05]
         [ 1.03542516e-02 -3.53018374e-03 -3.98914380e-05]]

        """

        if self._ncomp > 1:
            return np.array(self._data_copy).reshape(
                len(self._data_copy) // self._ncomp, self._ncomp
            )
        else:
            return np.array(self._data_copy)

    @data.setter
    def data(self, data):
        if self._is_property_field:
            if not isinstance(data[0], int) and not isinstance(data[0], np.int32):
                raise errors.InvalidTypeError("data", "list of int")
        else:
            if isinstance(data, (np.ndarray, np.generic)):
                if data.shape != self.shape and 0 != self.size:
                    raise ValueError(
                        f"An array of shape {self.shape} is expected and "
                        f"shape {data.shape} was input"
                    )
        if isinstance(data, (np.ndarray, np.generic)):
            self._data_copy = data.flatten().tolist()
        elif len(data) > 0 and isinstance(data, list):
            self._data_copy = np.array(data).flatten().tolist()
        else:
            self._data_copy = data

    @property
    def elementary_data_count(self):
        """Number of elementary data in the field.

        Returns
        -------
        int
           Number of elementary data in the field.

        """
        if hasattr(self, "_data_copy"):
            return len(self._data_copy) / self._ncomp
        else:
            return super().elementary_data_count

    @property
    def component_count(self):
        """Number of components in each elementary data of the field.

        Returns
        -------
        int
            Number of components in each elementary data of the field.
        """

        return self._ncomp

    @property
    def _data_pointer(self):
        """First index of each entity data in an array.

        Returns
        -------
        numpy.ndarray
            Array of first indexes of each entity data.
        """
        return np.array(self._data_pointer_copy)

    @property
    def _data_pointer_as_list(self):
        """First index of each entity data as a list.

        Returns
        -------
        List
            List of first indexes of each entity data.
        """
        return self._data_pointer_copy

    @_data_pointer.setter
    def _data_pointer(self, data):
        if isinstance(data, (np.ndarray, np.generic)):
            self._data_pointer_copy = data.tolist()
        else:
            self._data_pointer_copy = data
        if self._has_data_pointer == False and len(data) > 0:
            self._has_data_pointer = True

    @property
    def scoping_ids(self):
        """Scoping IDs of the field.

        Returns
        -------
        list
            List of integers representing the scoping IDs of the field.
        """
        return self._scoping_ids_copy

    @scoping_ids.setter
    def scoping_ids(self, data):
        self._scoping_ids_copy = data
        self._num_entities = len(data)

    def release_data(self):
        """Release the data."""
        super()._set_data(self._data_copy)
        super()._set_data_pointer(self._data_pointer_copy)
        super().scoping.ids = self._scoping_ids_copy

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        if tb is None:
            self.release_data()
        else:
            print(tb)

    def __del__(self):
        pass



from grpc._channel import _InactiveRpcError, _MultiThreadedRendezvous
from functools import wraps

_COMPLEX_PLOTTING_ERROR_MSG = """
Complex fields cannot be plotted. Use operators to get the amplitude
or the result at a defined sweeping phase before plotting.
"""

_FIELD_CONTAINER_PLOTTING_MSG = """"
This fields_container contains multiple fields.  Only one time-step
result can be plotted at a time. Extract a field with
``fields_container[index]``.
"""


class DpfVersionNotSupported(RuntimeError):
    """Error raised when the dpf-core/grpc-dpf python features are not
    supported by the DPF gRPC server version."""

    def __init__(self, version, msg=None):
        if msg is None:
            msg = "Feature not supported. Upgrade the server to "
            msg += str(version)
            msg += " version (or above)."
        RuntimeError.__init__(self, msg)


class DpfValueError(ValueError):
    """Error raised when a specific DPF error value must be defined."""

    def __init__(
        self, msg="A value that has been set leads to incorrect DPF behavior."
    ):
        ValueError.__init__(self, msg)


class InvalidTypeError(ValueError):
    """Error raised when a parameter has the wrong type."""

    def __init__(self, data_type, parameter_name):
        msg = (
            "A "
            + data_type
            + " must be used for the following parameter: "
            + parameter_name
            + "."
        )
        ValueError.__init__(self, msg)


class LocationError(ValueError):
    """Error raised when using an invalid location."""

    def __init__(self, msg="Invalid location"):
        ValueError.__init__(self, msg)


class ComplexPlottingError(ValueError):
    """Error raised when attempting to plot a field with complex data."""

    def __init__(self, msg=_COMPLEX_PLOTTING_ERROR_MSG):
        ValueError.__init__(self, msg)


class FieldContainerPlottingError(ValueError):
    """Error raised when attempting to plot a fields_container containing
    multiple fields."""

    def __init__(self, msg=_FIELD_CONTAINER_PLOTTING_MSG):
        ValueError.__init__(self, msg)


class InvalidANSYSVersionError(RuntimeError):
    """Error raised when the Ansys verion is invalid."""

    def __init__(self, msg=""):
        RuntimeError.__init__(self, msg)


class DPFServerException(Exception):
    """Error raised when the DPF server has encountered an error."""

    def __init__(self, msg=""):
        Exception.__init__(self, msg)


class DPFServerNullObject(Exception):
    """Error raised when the DPF server cannot find an object."""

    def __init__(self, msg=""):
        Exception.__init__(self, msg)


class InvalidPortError(OSError):
    """Error raised when used an invalid port when starting DPF."""

    def __init__(self, msg=""):
        OSError.__init__(self, msg)


def protect_grpc(func):
    """Capture gRPC exceptions and return a more succinct error message."""

    @wraps(func)
    def wrapper(*args, **kwargs):
        """Capture gRPC exceptions."""
        # Capture gRPC exceptions
        try:
            out = func(*args, **kwargs)
        except (_InactiveRpcError, _MultiThreadedRendezvous) as error:
            details = error.details()
            if "object is null in the dataBase" in details:
                raise DPFServerNullObject(details) from None
            raise DPFServerException(details) from None

        return out

    return wrapper



"""
serializer
==========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "serialization" category
"""

class serializer(Operator):
    """Take any input and serialize them in a file.

      available inputs:
        - file_path (str)
        - any_input1 (Any)
        - any_input2 (Any)

      available outputs:
        - file_path (str)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.serialization.serializer()

      >>> # Make input connections
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
      >>> my_any_input1 = dpf.Any()
      >>> op.inputs.any_input1.connect(my_any_input1)
      >>> my_any_input2 = dpf.Any()
      >>> op.inputs.any_input2.connect(my_any_input2)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.serialization.serializer(file_path=my_file_path,any_input1=my_any_input1,any_input2=my_any_input2)

      >>> # Get output data
      >>> result_file_path = op.outputs.file_path()"""
    def __init__(self, file_path=None, any_input1=None, any_input2=None, config=None, server=None):
        super().__init__(name="serializer", config = config, server = server)
        self._inputs = InputsSerializer(self)
        self._outputs = OutputsSerializer(self)
        if file_path !=None:
            self.inputs.file_path.connect(file_path)
        if any_input1 !=None:
            self.inputs.any_input1.connect(any_input1)
        if any_input2 !=None:
            self.inputs.any_input2.connect(any_input2)

    @staticmethod
    def _spec():
        spec = Specification(description="""Take any input and serialize them in a file.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "file_path", type_names=["string"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "any_input", type_names=["any"], optional=False, document="""any input"""), 
                                 2 : PinSpecification(name = "any_input", type_names=["any"], optional=False, document="""any input""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "file_path", type_names=["string"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "serializer")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSerializer 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsSerializer 
        """
        return super().outputs


#internal name: serializer
#scripting name: serializer
class InputsSerializer(_Inputs):
    """Intermediate class used to connect user inputs to serializer operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.serializer()
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
      >>> my_any_input1 = dpf.Any()
      >>> op.inputs.any_input1.connect(my_any_input1)
      >>> my_any_input2 = dpf.Any()
      >>> op.inputs.any_input2.connect(my_any_input2)
    """
    def __init__(self, op: Operator):
        super().__init__(serializer._spec().inputs, op)
        self._file_path = Input(serializer._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._file_path)
        self._any_input1 = Input(serializer._spec().input_pin(1), 1, op, 0) 
        self._inputs.append(self._any_input1)
        self._any_input2 = Input(serializer._spec().input_pin(2), 2, op, 1) 
        self._inputs.append(self._any_input2)

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator

        Parameters
        ----------
        my_file_path : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.serializer()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> #or
        >>> op.inputs.file_path(my_file_path)

        """
        return self._file_path

    @property
    def any_input1(self):
        """Allows to connect any_input1 input to the operator

        - pindoc: any input

        Parameters
        ----------
        my_any_input1 : Any, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.serializer()
        >>> op.inputs.any_input1.connect(my_any_input1)
        >>> #or
        >>> op.inputs.any_input1(my_any_input1)

        """
        return self._any_input1

    @property
    def any_input2(self):
        """Allows to connect any_input2 input to the operator

        - pindoc: any input

        Parameters
        ----------
        my_any_input2 : Any, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.serializer()
        >>> op.inputs.any_input2.connect(my_any_input2)
        >>> #or
        >>> op.inputs.any_input2(my_any_input2)

        """
        return self._any_input2

class OutputsSerializer(_Outputs):
    """Intermediate class used to get outputs from serializer operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.serializer()
      >>> # Connect inputs : op.inputs. ...
      >>> result_file_path = op.outputs.file_path()
    """
    def __init__(self, op: Operator):
        super().__init__(serializer._spec().outputs, op)
        self._file_path = Output(serializer._spec().output_pin(0), 0, op) 
        self._outputs.append(self._file_path)

    @property
    def file_path(self):
        """Allows to get file_path output of the operator


        Returns
        ----------
        my_file_path : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.serializer()
        >>> # Connect inputs : op.inputs. ...
        >>> result_file_path = op.outputs.file_path() 
        """
        return self._file_path




"""
elemental_mean
==============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "averaging" category
"""

class elemental_mean(Operator):
    """Computes the average of a multi-entity fields, (ElementalNodal -> Elemental), (NodalElemental -> Nodal).

      available inputs:
        - field (Field)
        - collapse_shell_layers (bool) (optional)
        - force_averaging (bool) (optional)
        - scoping (Scoping) (optional)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.averaging.elemental_mean()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_collapse_shell_layers = bool()
      >>> op.inputs.collapse_shell_layers.connect(my_collapse_shell_layers)
      >>> my_force_averaging = bool()
      >>> op.inputs.force_averaging.connect(my_force_averaging)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.averaging.elemental_mean(field=my_field,collapse_shell_layers=my_collapse_shell_layers,force_averaging=my_force_averaging,scoping=my_scoping)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, field=None, collapse_shell_layers=None, force_averaging=None, scoping=None, config=None, server=None):
        super().__init__(name="entity_average", config = config, server = server)
        self._inputs = InputsElementalMean(self)
        self._outputs = OutputsElementalMean(self)
        if field !=None:
            self.inputs.field.connect(field)
        if collapse_shell_layers !=None:
            self.inputs.collapse_shell_layers.connect(collapse_shell_layers)
        if force_averaging !=None:
            self.inputs.force_averaging.connect(force_averaging)
        if scoping !=None:
            self.inputs.scoping.connect(scoping)

    @staticmethod
    def _spec():
        spec = Specification(description="""Computes the average of a multi-entity fields, (ElementalNodal -> Elemental), (NodalElemental -> Nodal).""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "collapse_shell_layers", type_names=["bool"], optional=True, document="""if true shell layers are averaged as well (default is false)"""), 
                                 2 : PinSpecification(name = "force_averaging", type_names=["bool"], optional=True, document="""if true you average, if false you just sum"""), 
                                 3 : PinSpecification(name = "scoping", type_names=["scoping"], optional=True, document="""average only on these elements, if it is scoping container, the label must correspond to the one of the fields container""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "entity_average")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsElementalMean 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsElementalMean 
        """
        return super().outputs


#internal name: entity_average
#scripting name: elemental_mean
class InputsElementalMean(_Inputs):
    """Intermediate class used to connect user inputs to elemental_mean operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.elemental_mean()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_collapse_shell_layers = bool()
      >>> op.inputs.collapse_shell_layers.connect(my_collapse_shell_layers)
      >>> my_force_averaging = bool()
      >>> op.inputs.force_averaging.connect(my_force_averaging)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
    """
    def __init__(self, op: Operator):
        super().__init__(elemental_mean._spec().inputs, op)
        self._field = Input(elemental_mean._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)
        self._collapse_shell_layers = Input(elemental_mean._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._collapse_shell_layers)
        self._force_averaging = Input(elemental_mean._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._force_averaging)
        self._scoping = Input(elemental_mean._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._scoping)

    @property
    def field(self):
        """Allows to connect field input to the operator

        Parameters
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_mean()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

    @property
    def collapse_shell_layers(self):
        """Allows to connect collapse_shell_layers input to the operator

        - pindoc: if true shell layers are averaged as well (default is false)

        Parameters
        ----------
        my_collapse_shell_layers : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_mean()
        >>> op.inputs.collapse_shell_layers.connect(my_collapse_shell_layers)
        >>> #or
        >>> op.inputs.collapse_shell_layers(my_collapse_shell_layers)

        """
        return self._collapse_shell_layers

    @property
    def force_averaging(self):
        """Allows to connect force_averaging input to the operator

        - pindoc: if true you average, if false you just sum

        Parameters
        ----------
        my_force_averaging : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_mean()
        >>> op.inputs.force_averaging.connect(my_force_averaging)
        >>> #or
        >>> op.inputs.force_averaging(my_force_averaging)

        """
        return self._force_averaging

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator

        - pindoc: average only on these elements, if it is scoping container, the label must correspond to the one of the fields container

        Parameters
        ----------
        my_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_mean()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> #or
        >>> op.inputs.scoping(my_scoping)

        """
        return self._scoping

class OutputsElementalMean(_Outputs):
    """Intermediate class used to get outputs from elemental_mean operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.elemental_mean()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(elemental_mean._spec().outputs, op)
        self._field = Output(elemental_mean._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_mean()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""
Dimensionality
==============
"""

from ansys.grpc.dpf import field_definition_pb2
from ansys.dpf.core.common import natures


class Dimensionality:
    """Represents the dimensionality of a field.

        This class reads a list of dimension vectors and creates a dimensionality message.

    Parameters
    ----------
    dim_vec : list
        List of integers for the dimension vectors. There is a 1D vector for a
        scalar or 3D vector. There is a 2D vector for a matrix. For example:

        * ``[1]``: scalar
        * ``[3]``: 3d vector
        * ``[3, 3]``: ``3 x 3`` matrix
    nature : :class:`ansys.dpf.core.common.natures`

    """

    def __init__(self, dim_vec, nature: natures):
        self.dim = dim_vec
        self.nature = nature

    def _parse_dim_to_message(self):
        message = field_definition_pb2.Dimensionality()
        message.size.extend(self.dim)
        message.nature = self.nature.value
        return message

    def __str__(self):
        return str(self.dim) + " " + self.nature.name

    @staticmethod
    def scalar_dim():
        """Retrieve the dimensionality of a scalar field.

        Returns
        -------
        :class:`ansys.dpf.core.dimensionnality.Dimensionality`
            Dimensionality of the scalar field.
        """
        return Dimensionality([1], natures.scalar)

    @staticmethod
    def vector_dim(size):
        """Retrieve the dimensionality of a vector field of a given size.

        Parameters
        ----------
        size : int
            Number of components by entity.

        Returns
        -------
        :class:`ansys.dpf.core.dimensionnality.Dimensionality`
            Dimensionality of the vector field.
        """
        return Dimensionality([size], natures.vector)

    @staticmethod
    def vector_3d_dim():
        """Retrieve the dimensionality of a three-dimension vector field.

        Returns
        -------
        :class:`ansys.dpf.core.dimensionnality.Dimensionality`
            Dimensionality of the three-dimension vector field.
        """
        return Dimensionality([3], natures.vector)

    @staticmethod
    def tensor_dim():
        """Retrieve the dimensionality of a symmetrical ``3 x 3`` tensor field.

        Returns
        -------
        :class:`ansys.dpf.core.dimensionnality.Dimensionality`
            Dimensionality of the symmetrical ``3 x 3`` tensor field.
        """
        return Dimensionality([3, 3], natures.symmatrix)



"""
prns_to_field
=============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from mapdlOperatorsCore plugin, from "result" category
"""

class prns_to_field(Operator):
    """Read the presol of nodal field generated file from mapdl.

      available inputs:
        - filepath (str)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.prns_to_field()

      >>> # Make input connections
      >>> my_filepath = str()
      >>> op.inputs.filepath.connect(my_filepath)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.prns_to_field(filepath=my_filepath)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, filepath=None, config=None, server=None):
        super().__init__(name="PRNS_Reader", config = config, server = server)
        self._inputs = InputsPrnsToField(self)
        self._outputs = OutputsPrnsToField(self)
        if filepath !=None:
            self.inputs.filepath.connect(filepath)

    @staticmethod
    def _spec():
        spec = Specification(description="""Read the presol of nodal field generated file from mapdl.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "filepath", type_names=["string"], optional=False, document="""filepath""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "PRNS_Reader")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsPrnsToField 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsPrnsToField 
        """
        return super().outputs


#internal name: PRNS_Reader
#scripting name: prns_to_field
class InputsPrnsToField(_Inputs):
    """Intermediate class used to connect user inputs to prns_to_field operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.prns_to_field()
      >>> my_filepath = str()
      >>> op.inputs.filepath.connect(my_filepath)
    """
    def __init__(self, op: Operator):
        super().__init__(prns_to_field._spec().inputs, op)
        self._filepath = Input(prns_to_field._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._filepath)

    @property
    def filepath(self):
        """Allows to connect filepath input to the operator

        - pindoc: filepath

        Parameters
        ----------
        my_filepath : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.prns_to_field()
        >>> op.inputs.filepath.connect(my_filepath)
        >>> #or
        >>> op.inputs.filepath(my_filepath)

        """
        return self._filepath

class OutputsPrnsToField(_Outputs):
    """Intermediate class used to get outputs from prns_to_field operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.prns_to_field()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(prns_to_field._spec().outputs, op)
        self._field = Output(prns_to_field._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.prns_to_field()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""
boundary_condition_provider
===========================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "metadata" category
"""

class boundary_condition_provider(Operator):
    """Read boundary conditions from the results files contained in the streams or data sources.

      available inputs:
        - streams_container (StreamsContainer) (optional)
        - data_sources (DataSources)

      available outputs:
        - results_info (Field ,FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.metadata.boundary_condition_provider()

      >>> # Make input connections
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.metadata.boundary_condition_provider(streams_container=my_streams_container,data_sources=my_data_sources)

      >>> # Get output data
      >>> result_results_info = op.outputs.results_info()"""
    def __init__(self, streams_container=None, data_sources=None, config=None, server=None):
        super().__init__(name="boundary_conditions", config = config, server = server)
        self._inputs = InputsBoundaryConditionProvider(self)
        self._outputs = OutputsBoundaryConditionProvider(self)
        if streams_container !=None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        spec = Specification(description="""Read boundary conditions from the results files contained in the streams or data sources.""",
                             map_input_pin_spec={
                                 3 : PinSpecification(name = "streams_container", type_names=["streams_container"], optional=True, document=""""""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "results_info", type_names=["field","fields_container"], optional=False, document="""results info""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "boundary_conditions")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsBoundaryConditionProvider 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsBoundaryConditionProvider 
        """
        return super().outputs


#internal name: boundary_conditions
#scripting name: boundary_condition_provider
class InputsBoundaryConditionProvider(_Inputs):
    """Intermediate class used to connect user inputs to boundary_condition_provider operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.metadata.boundary_condition_provider()
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
    """
    def __init__(self, op: Operator):
        super().__init__(boundary_condition_provider._spec().inputs, op)
        self._streams_container = Input(boundary_condition_provider._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._streams_container)
        self._data_sources = Input(boundary_condition_provider._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator

        Parameters
        ----------
        my_streams_container : StreamsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.boundary_condition_provider()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> #or
        >>> op.inputs.streams_container(my_streams_container)

        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.boundary_condition_provider()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

class OutputsBoundaryConditionProvider(_Outputs):
    """Intermediate class used to get outputs from boundary_condition_provider operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.metadata.boundary_condition_provider()
      >>> # Connect inputs : op.inputs. ...
      >>> result_results_info = op.outputs.results_info()
    """
    def __init__(self, op: Operator):
        super().__init__(boundary_condition_provider._spec().outputs, op)
        self.results_info_as_field = Output( _modify_output_spec_with_one_type(boundary_condition_provider._spec().output_pin(0), "field"), 0, op) 
        self._outputs.append(self.results_info_as_field)
        self.results_info_as_fields_container = Output( _modify_output_spec_with_one_type(boundary_condition_provider._spec().output_pin(0), "fields_container"), 0, op) 
        self._outputs.append(self.results_info_as_fields_container)




"""
from_mesh
=========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "scoping" category
"""

class from_mesh(Operator):
    """Provides the entire mesh scoping based on the requested location

      available inputs:
        - mesh (MeshedRegion)
        - requested_location (str) (optional)

      available outputs:
        - scoping (Scoping)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.scoping.from_mesh()

      >>> # Make input connections
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.scoping.from_mesh(mesh=my_mesh,requested_location=my_requested_location)

      >>> # Get output data
      >>> result_scoping = op.outputs.scoping()"""
    def __init__(self, mesh=None, requested_location=None, config=None, server=None):
        super().__init__(name="MeshScopingProvider", config = config, server = server)
        self._inputs = InputsFromMesh(self)
        self._outputs = OutputsFromMesh(self)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if requested_location !=None:
            self.inputs.requested_location.connect(requested_location)

    @staticmethod
    def _spec():
        spec = Specification(description="""Provides the entire mesh scoping based on the requested location""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "requested_location", type_names=["string"], optional=True, document="""if nothing the operator returns the nodes scoping, possible locations are: Nodal or Elemental""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "scoping", type_names=["scoping"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "MeshScopingProvider")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFromMesh 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsFromMesh 
        """
        return super().outputs


#internal name: MeshScopingProvider
#scripting name: from_mesh
class InputsFromMesh(_Inputs):
    """Intermediate class used to connect user inputs to from_mesh operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.from_mesh()
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
    """
    def __init__(self, op: Operator):
        super().__init__(from_mesh._spec().inputs, op)
        self._mesh = Input(from_mesh._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._mesh)
        self._requested_location = Input(from_mesh._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._requested_location)

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.from_mesh()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator

        - pindoc: if nothing the operator returns the nodes scoping, possible locations are: Nodal or Elemental

        Parameters
        ----------
        my_requested_location : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.from_mesh()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> #or
        >>> op.inputs.requested_location(my_requested_location)

        """
        return self._requested_location

class OutputsFromMesh(_Outputs):
    """Intermediate class used to get outputs from from_mesh operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.from_mesh()
      >>> # Connect inputs : op.inputs. ...
      >>> result_scoping = op.outputs.scoping()
    """
    def __init__(self, op: Operator):
        super().__init__(from_mesh._spec().outputs, op)
        self._scoping = Output(from_mesh._spec().output_pin(0), 0, op) 
        self._outputs.append(self._scoping)

    @property
    def scoping(self):
        """Allows to get scoping output of the operator


        Returns
        ----------
        my_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.from_mesh()
        >>> # Connect inputs : op.inputs. ...
        >>> result_scoping = op.outputs.scoping() 
        """
        return self._scoping




"""
mesh_cut
========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from meshOperatorsCore plugin, from "mesh" category
"""

class mesh_cut(Operator):
    """Extracts a skin of the mesh in triangles (2D elements) in a new meshed region

      available inputs:
        - field (Field)
        - iso_value (float)
        - closed_surface (float)
        - slice_surfaces (bool)

      available outputs:
        - mesh (MeshedRegion)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.mesh.mesh_cut()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_iso_value = float()
      >>> op.inputs.iso_value.connect(my_iso_value)
      >>> my_closed_surface = float()
      >>> op.inputs.closed_surface.connect(my_closed_surface)
      >>> my_slice_surfaces = bool()
      >>> op.inputs.slice_surfaces.connect(my_slice_surfaces)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.mesh.mesh_cut(field=my_field,iso_value=my_iso_value,closed_surface=my_closed_surface,slice_surfaces=my_slice_surfaces)

      >>> # Get output data
      >>> result_mesh = op.outputs.mesh()"""
    def __init__(self, field=None, iso_value=None, closed_surface=None, slice_surfaces=None, config=None, server=None):
        super().__init__(name="mesh_cut", config = config, server = server)
        self._inputs = InputsMeshCut(self)
        self._outputs = OutputsMeshCut(self)
        if field !=None:
            self.inputs.field.connect(field)
        if iso_value !=None:
            self.inputs.iso_value.connect(iso_value)
        if closed_surface !=None:
            self.inputs.closed_surface.connect(closed_surface)
        if slice_surfaces !=None:
            self.inputs.slice_surfaces.connect(slice_surfaces)

    @staticmethod
    def _spec():
        spec = Specification(description="""Extracts a skin of the mesh in triangles (2D elements) in a new meshed region""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "iso_value", type_names=["double"], optional=False, document="""iso value"""), 
                                 3 : PinSpecification(name = "closed_surface", type_names=["double"], optional=False, document="""1: closed surface, 0:iso surface"""), 
                                 4 : PinSpecification(name = "slice_surfaces", type_names=["bool"], optional=False, document="""true: slicing will also take into account shell and 2D elements, false: sliicing will ignore shell and 2D elements. default is true""")},
                             map_output_pin_spec={
                                 2 : PinSpecification(name = "mesh", type_names=["meshed_region"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "mesh_cut")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMeshCut 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsMeshCut 
        """
        return super().outputs


#internal name: mesh_cut
#scripting name: mesh_cut
class InputsMeshCut(_Inputs):
    """Intermediate class used to connect user inputs to mesh_cut operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.mesh_cut()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_iso_value = float()
      >>> op.inputs.iso_value.connect(my_iso_value)
      >>> my_closed_surface = float()
      >>> op.inputs.closed_surface.connect(my_closed_surface)
      >>> my_slice_surfaces = bool()
      >>> op.inputs.slice_surfaces.connect(my_slice_surfaces)
    """
    def __init__(self, op: Operator):
        super().__init__(mesh_cut._spec().inputs, op)
        self._field = Input(mesh_cut._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)
        self._iso_value = Input(mesh_cut._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._iso_value)
        self._closed_surface = Input(mesh_cut._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._closed_surface)
        self._slice_surfaces = Input(mesh_cut._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._slice_surfaces)

    @property
    def field(self):
        """Allows to connect field input to the operator

        Parameters
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.mesh_cut()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

    @property
    def iso_value(self):
        """Allows to connect iso_value input to the operator

        - pindoc: iso value

        Parameters
        ----------
        my_iso_value : float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.mesh_cut()
        >>> op.inputs.iso_value.connect(my_iso_value)
        >>> #or
        >>> op.inputs.iso_value(my_iso_value)

        """
        return self._iso_value

    @property
    def closed_surface(self):
        """Allows to connect closed_surface input to the operator

        - pindoc: 1: closed surface, 0:iso surface

        Parameters
        ----------
        my_closed_surface : float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.mesh_cut()
        >>> op.inputs.closed_surface.connect(my_closed_surface)
        >>> #or
        >>> op.inputs.closed_surface(my_closed_surface)

        """
        return self._closed_surface

    @property
    def slice_surfaces(self):
        """Allows to connect slice_surfaces input to the operator

        - pindoc: true: slicing will also take into account shell and 2D elements, false: sliicing will ignore shell and 2D elements. default is true

        Parameters
        ----------
        my_slice_surfaces : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.mesh_cut()
        >>> op.inputs.slice_surfaces.connect(my_slice_surfaces)
        >>> #or
        >>> op.inputs.slice_surfaces(my_slice_surfaces)

        """
        return self._slice_surfaces

class OutputsMeshCut(_Outputs):
    """Intermediate class used to get outputs from mesh_cut operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.mesh_cut()
      >>> # Connect inputs : op.inputs. ...
      >>> result_mesh = op.outputs.mesh()
    """
    def __init__(self, op: Operator):
        super().__init__(mesh_cut._spec().outputs, op)
        self._mesh = Output(mesh_cut._spec().output_pin(2), 2, op) 
        self._outputs.append(self._mesh)

    @property
    def mesh(self):
        """Allows to get mesh output of the operator


        Returns
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.mesh_cut()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh = op.outputs.mesh() 
        """
        return self._mesh




"""
streams_provider
================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "metadata" category
"""

class streams_provider(Operator):
    """Creates streams (files with cache) from the data sources.

      available inputs:
        - data_sources (DataSources)

      available outputs:
        - streams_container (StreamsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.metadata.streams_provider()

      >>> # Make input connections
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.metadata.streams_provider(data_sources=my_data_sources)

      >>> # Get output data
      >>> result_streams_container = op.outputs.streams_container()"""
    def __init__(self, data_sources=None, config=None, server=None):
        super().__init__(name="stream_provider", config = config, server = server)
        self._inputs = InputsStreamsProvider(self)
        self._outputs = OutputsStreamsProvider(self)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        spec = Specification(description="""Creates streams (files with cache) from the data sources.""",
                             map_input_pin_spec={
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "streams_container", type_names=["streams_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "stream_provider")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsStreamsProvider 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsStreamsProvider 
        """
        return super().outputs


#internal name: stream_provider
#scripting name: streams_provider
class InputsStreamsProvider(_Inputs):
    """Intermediate class used to connect user inputs to streams_provider operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.metadata.streams_provider()
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
    """
    def __init__(self, op: Operator):
        super().__init__(streams_provider._spec().inputs, op)
        self._data_sources = Input(streams_provider._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.streams_provider()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

class OutputsStreamsProvider(_Outputs):
    """Intermediate class used to get outputs from streams_provider operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.metadata.streams_provider()
      >>> # Connect inputs : op.inputs. ...
      >>> result_streams_container = op.outputs.streams_container()
    """
    def __init__(self, op: Operator):
        super().__init__(streams_provider._spec().outputs, op)
        self._streams_container = Output(streams_provider._spec().output_pin(0), 0, op) 
        self._outputs.append(self._streams_container)

    @property
    def streams_container(self):
        """Allows to get streams_container output of the operator


        Returns
        ----------
        my_streams_container : StreamsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.streams_provider()
        >>> # Connect inputs : op.inputs. ...
        >>> result_streams_container = op.outputs.streams_container() 
        """
        return self._streams_container




"""
.. _ref_operator:

Operator
========
Provides an interface to the underlying gRPC operator.
"""

import functools
import logging

from ansys.dpf.core import server as serverlib
from ansys.dpf.core.config import Config
from ansys.dpf.core.errors import protect_grpc
from ansys.dpf.core.inputs import Inputs
from ansys.dpf.core.mapping_types import types
from ansys.dpf.core.outputs import Output, Outputs, _Outputs
from ansys.grpc.dpf import base_pb2, operator_pb2, operator_pb2_grpc

LOG = logging.getLogger(__name__)
LOG.setLevel("DEBUG")


class Operator:
    """Represents an operator, which is an elementary operation.

    The operator is the only object used to create and transform
    data. When the operator is evaluated, it processes the
    input information to compute its output with respect to its
    description.

    Parameters
    ----------
    name : str
        Name of the operator. For example, ``"U"``. You can use the
        ``"html_doc"`` operator to retrieve a list of existing operators.
    config : ansys.dpf.core.Config, optional
        The default is ``None``.
    server : server.DPFServer, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.

    Examples
    --------
    Create an operator from the library of operators.

    >>> from ansys.dpf import core as dpf
    >>> disp_oper = dpf.operators.result.displacement()

    Create an operator from a model.

    >>> from ansys.dpf.core import Model
    >>> from ansys.dpf.core import examples
    >>> model = Model(examples.static_rst)
    >>> disp_oper = model.results.displacement()

    """

    def __init__(self, name, config=None, server=None):
        """Initialize the operator with its name by connecting to a stub."""
        if server is None:
            server = serverlib._global_server()

        self._server = server

        self.name = name
        self._stub = self._connect()

        self._message = None
        self._description = None
        self._inputs = None
        self._outputs = None

        self.__send_init_request(config)

        # add dynamic inputs
        if len(self._message.spec.map_input_pin_spec) > 0 and self._inputs == None:
            self._inputs = Inputs(self._message.spec.map_input_pin_spec, self)
        if len(self._message.spec.map_output_pin_spec) != 0 and self._outputs == None:
            self._outputs = Outputs(self._message.spec.map_output_pin_spec, self)

        self._description = self._message.spec.description

    def _add_sub_res_operators(self, sub_results):
        """Dynamically add operators for instantiating subresults.

        Subresults for new operators are connected to the parent
        operator's inputs when created but are then completely
        independent of them.

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> model = Model(examples.static_rst)
        >>> disp_oper = model.results.displacement()
        >>> disp_oper = model.results.displacement()
        >>> disp_x = model.results.displacement().X()
        >>> disp_y = model.results.displacement().Y()
        >>> disp_z = model.results.displacement().Z()

        """

        for result_type in sub_results:
            bound_method = self._sub_result_op.__get__(self, self.__class__)
            method2 = functools.partial(bound_method, name=result_type["operator name"])
            setattr(self, result_type["name"], method2)

    @protect_grpc
    def connect(self, pin, inpt, pin_out=0):
        """Connect an input on the operator using a pin number.

        Parameters
        ----------
        pin : int
            Number of the input pin.
        inpt : str, int, double, bool, list of int, list of doubles,
               Field, FieldsContainer, Scoping, ScopingsContainer, MeshedRegion,
               MeshesContainer, DataSources, Operator
            Object to connect to.
        pin_out : int, optional
            If the input is an operator, the output pin of the input operator. The
            default is ``0``.

        Examples
        --------
        Compute the minimum of displacement by chaining the ``"U"``
        and ``"min_max_fc"`` operators.

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> data_src = dpf.DataSources(examples.multishells_rst)
        >>> disp_op = dpf.operators.result.displacement()
        >>> disp_op.inputs.data_sources(data_src)
        >>> max_fc_op = dpf.operators.min_max.min_max_fc()
        >>> max_fc_op.inputs.connect(disp_op.outputs)
        >>> max_field = max_fc_op.outputs.field_max()
        >>> max_field.data
        array([[0.59428386, 0.00201751, 0.0006032 ]])

        """

        request = operator_pb2.UpdateRequest()
        request.op.CopyFrom(self._message)
        request.pin = pin
        _fillConnectionRequestMessage(request, inpt, pin_out)
        if inpt is self:
            raise ValueError("Cannot connect to itself.")
        self._stub.Update(request)

    @protect_grpc
    def get_output(self, pin=0, output_type=None):
        """Retrieve the output of the operator on the pin number.

        Parameters
        ----------
        pin : int, optional
            Number of the output pin. The default is ``0``.
        output_type : :class:`ansys.dpf.core.common.types`, optional
            Requested type of the output. The default is ``None``.

        Returns
        -------
        type
            Output of the operator.
        """

        request = operator_pb2.OperatorEvaluationRequest()
        request.op.CopyFrom(self._message)
        request.pin = pin

        if output_type is not None:
            _write_output_type_to_proto_style(output_type, request)
            out = self._stub.Get(request)
            return _convertOutputMessageToPythonInstance(out, output_type, self._server)
        else:
            request.type = base_pb2.Type.Value("RUN")
            return self._stub.Get(request)

    @property
    def config(self):
        """Copy of the operator's current configuration.

        You can modify the copy of the configuration and then use ``operator.config = new_config``
        or create an operator with the new configuration as a parameter.

        Returns
        ----------
        :class:`ansys.dpf.core.config.Config`
            Copy of the operator's current configuration.
        """

        out = self._stub.List(self._message)
        config = out.config
        return Config(config=config, server=self._server)

    @config.setter
    def config(self, value):
        """Change the configuration of the operator.

        If the operator is up to date, changing the configuration
        doesn't make it not up to date.

        Parameters
        ----------
        value : Config
        """
        request = operator_pb2.UpdateConfigRequest()
        request.op.CopyFrom(self._message)
        request.config.CopyFrom(value._message)
        self._stub.UpdateConfig(request)

    @property
    def inputs(self):
        """Inputs connected to the operator.

        Returns
        --------
        :class:`ansys.dpf.core.inputs`
            Inputs connected to the operator.

        Examples
        --------
        Use the displacement operator.

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> data_src = dpf.DataSources(examples.multishells_rst)
        >>> disp_op = dpf.operators.result.displacement()
        >>> disp_op.inputs.data_sources(data_src)

        """

        return self._inputs

    @property
    def outputs(self):
        """Outputs from the operator's evaluation.

        Returns
        --------
        :class:`ansys.dpf.core.outputs`
            Outputs from the operator's evaluation.

        Examples
        --------
        Use the displacement operator.

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> data_src = dpf.DataSources(examples.multishells_rst)
        >>> disp_op = dpf.operators.result.displacement()
        >>> disp_op.inputs.data_sources(data_src)
        >>> disp_fc = disp_op.outputs.fields_container()

        """
        return self._outputs

    @staticmethod
    def default_config(name, server=None):
        """Retrieve the default configuration for an operator.

        You can change the copy of the default configuration to meet your needs
        before instantiating the operator.

        Parameters
        ----------
        name : str
            Name of the operator.  For example ``"U"``. You can use the
            ``"html_doc"`` operator to retrieve a list of existing operators.
        server : server.DPFServer, optional
            Server with the channel connected to the remote or local instance. The
            default is ``None``, in which case an attempt is made to use the global
            server.

        Returns
        -------
        :class"`ansys.dpf.core.config.Config`
            Default configuration for the operator.

        """
        return Config(operator_name=name, server=server)

    def _connect(self):
        """Connect to the gRPC service."""
        return operator_pb2_grpc.OperatorServiceStub(self._server.channel)

    def __del__(self):
        try:
            self._stub.Delete(self._message)
        except:
            pass

    def __str__(self):
        """Describe the entity.

        Returns
        -------
        str
            Description of the entity.
        """
        from ansys.dpf.core.core import _description

        return _description(self._message, self._server)

    def run(self):
        """Evaluate this operator."""
        self.get_output()

    def eval(self, pin=None):
        """Evaluate this operator.

        Parameters
        ----------
        pin : int
            Number of the output pin. The default is ``None``.

        Returns
        -------
        output : FieldsContainer, Field, MeshedRegion, Scoping
            Returns the first output of the operator by default and the output of a
            given pin when specified. Or, it only evaluates the operator without output.

        Examples
        --------
        Use the ``eval`` method.

        >>> from ansys.dpf import core as dpf
        >>> import ansys.dpf.core.operators.math as math
        >>> from ansys.dpf.core import examples
        >>> data_src = dpf.DataSources(examples.multishells_rst)
        >>> disp_op = dpf.operators.result.displacement()
        >>> disp_op.inputs.data_sources(data_src)
        >>> normfc = math.norm_fc(disp_op).eval()

        """

        if not pin:
            if self.outputs != None and len(self.outputs._outputs) > 0:
                return self.outputs._outputs[0]()
            else:
                self.run()
        else:
            for output in self.outputs._outputs:
                if output._pin == pin:
                    return output()

    def _find_outputs_corresponding_pins(
        self, type_names, inpt, pin, corresponding_pins
    ):
        from ansys.dpf.core.results import Result
        for python_name in type_names:
            # appears to be an issue on Linux.  This check is here
            # because cpp mappings are a single type mapping and
            # sometimes the spec contains 'B' instead of 'bool'
            if python_name == "B":
                python_name = "bool"

            if type(inpt).__name__ == python_name:
                corresponding_pins.append(pin)
            elif isinstance(inpt, (_Outputs, Operator, Result)):
                if isinstance(inpt, Operator):
                    output_pin_available = inpt.outputs._get_given_output([python_name])
                elif isinstance(inpt, Result):
                    output_pin_available = inpt().outputs._get_given_output([python_name])
                else:
                    output_pin_available = inpt._get_given_output([python_name])
                for outputpin in output_pin_available:
                    corresponding_pins.append((pin, outputpin))
            elif isinstance(inpt, Output):
                for inpttype in inpt._python_expected_types:
                    if inpttype == python_name:
                        corresponding_pins.append(pin)
                if python_name == "Any":
                    corresponding_pins.append(pin)
            elif python_name == "Any":
                corresponding_pins.append(pin)

    @protect_grpc
    def _sub_result_op(self, name):
        op = Operator(name)
        if self.inputs is not None:
            for key in self.inputs._connected_inputs:
                inpt = self.inputs._connected_inputs[key]
                if type(inpt).__name__ == "dict":
                    for keyout in inpt:
                        op.connect(key, inpt[keyout], keyout)
                else:
                    op.connect(key, inpt)
        return op

    @protect_grpc
    def __send_init_request(self, config=None):
        request = operator_pb2.CreateOperatorRequest()
        request.name = self.name
        if config:
            request.config.CopyFrom(config._message)
        self._message = self._stub.Create(request)

    def __add__(self, fields_b):
        """Add two fields or two fields containers.

        Returns
        -------
        add : operators.math.add_fc
        """
        from ansys.dpf.core import dpf_operator, operators

        if hasattr(operators, "math") and hasattr(operators.math, "add_fc"):
            op = operators.math.add_fc(self, fields_b, server=self._server)
        else:
            op = dpf_operator.Operator("add_fc", server=self._server)
            op.connect(0, self)
            op.connect(1, fields_b)
        return op

    def __sub__(self, fields_b):
        """Subtract two fields or two fields containers.

        Returns
        -------
        minus : operators.math.minus_fc
        """
        from ansys.dpf.core import dpf_operator, operators

        if hasattr(operators, "math") and hasattr(operators.math, "minus_fc"):
            op = operators.math.minus_fc(server=self._server)
        else:
            op = dpf_operator.Operator("minus_fc", server=self._server)
        op.connect(0, self)
        op.connect(1, fields_b)
        return op

    def __pow__(self, value):
        if value != 2:
            raise ValueError('Only the value "2" is suppported.')
        from ansys.dpf.core import dpf_operator, operators

        if hasattr(operators, "math") and hasattr(operators.math, "sqr_fc"):
            op = operators.math.sqr_fc(server=self._server)
        else:
            op = dpf_operator.Operator("sqr_fc", server=self._server)
        op.connect(0, self)
        op.connect(1, value)
        return op

    def __mul__(self, value):
        """Multiply two fields or two fields containers.

        Returns
        -------
        mul : operators.math.generalized_inner_product_fc
        """
        from ansys.dpf.core import dpf_operator, operators

        if hasattr(operators, "math") and hasattr(
            operators.math, "generalized_inner_product_fc"
        ):
            op = operators.math.generalized_inner_product_fc(server=self._server)
        else:
            op = dpf_operator.Operator(
                "generalized_inner_product_fc", server=self._server
            )
        op.connect(0, self)
        op.connect(1, value)
        return op

    def __truediv__(self, inpt):
        if isinstance(inpt, Operator):
            op = Operator("div")
            op.connect(0, self, 0)
            op.connect(1, inpt, 0)
        elif isinstance(inpt, float):
            op = Operator("scale")
            op.connect(0, self, 0)
            op.connect(1, 1.0 / inpt)
        return op


def _write_output_type_to_proto_style(output_type, request):
    subtype = ""
    stype = ""
    if hasattr(output_type, "name"):
        if output_type == types.fields_container:
            stype = "collection"
            subtype = "field"
        elif output_type == types.scopings_container:
            stype = "collection"
            subtype = "scoping"
        elif output_type == types.meshes_container:
            stype = "collection"
            subtype = "meshed_region"
        else:
            stype = output_type.name
    elif isinstance(output_type, list):
        stype = output_type[0]
        subtype = output_type[1]
    else:
        stype = output_type
    request.type = base_pb2.Type.Value(stype.upper())
    if subtype != "":
        request.subtype = base_pb2.Type.Value(subtype.upper())


def _convertOutputMessageToPythonInstance(out, output_type, server):
    from ansys.dpf.core import (
        cyclic_support,
        data_sources,
        field,
        fields_container,
        meshed_region,
        meshes_container,
        property_field,
        result_info,
        scoping,
        scopings_container,
        time_freq_support,
    )

    if out.HasField("str"):
        return out.str
    elif out.HasField("int"):
        return out.int
    elif out.HasField("double"):
        return out.double
    elif out.HasField("bool"):
        return out.bool
    elif out.HasField("field"):
        toconvert = out.field
        if toconvert.datatype == "int":
            return property_field.PropertyField(server=server, property_field=toconvert)
        else:
            return field.Field(server=server, field=toconvert)
    elif out.HasField("collection"):
        toconvert = out.collection
        if output_type == types.fields_container:
            return fields_container.FieldsContainer(
                server=server, fields_container=toconvert
            )
        elif output_type == types.scopings_container:
            return scopings_container.ScopingsContainer(
                server=server, scopings_container=toconvert
            )
        elif output_type == types.meshes_container:
            return meshes_container.MeshesContainer(
                server=server, meshes_container=toconvert
            )
    elif out.HasField("scoping"):
        toconvert = out.scoping
        return scoping.Scoping(scoping=toconvert, server=server)
    elif out.HasField("mesh"):
        toconvert = out.mesh
        return meshed_region.MeshedRegion(mesh=toconvert, server=server)
    elif out.HasField("result_info"):
        toconvert = out.result_info
        return result_info.ResultInfo(result_info=toconvert, server=server)
    elif out.HasField("time_freq_support"):
        toconvert = out.time_freq_support
        return time_freq_support.TimeFreqSupport(
            server=server, time_freq_support=toconvert
        )
    elif out.HasField("data_sources"):
        toconvert = out.data_sources
        return data_sources.DataSources(server=server, data_sources=toconvert)
    elif out.HasField("cyc_support"):
        toconvert = out.cyc_support
        return cyclic_support.CyclicSupport(server=server, cyclic_support=toconvert)


def _fillConnectionRequestMessage(request, inpt, pin_out=0):
    from ansys.dpf.core import (
        collection,
        cyclic_support,
        data_sources,
        field_base,
        meshed_region,
        model,
        scoping,
    )

    if isinstance(inpt, str):
        request.str = inpt
    elif isinstance(inpt, bool):
        request.bool = inpt
    elif isinstance(inpt, int):
        request.int = inpt
    elif isinstance(inpt, float):
        request.double = inpt
    elif isinstance(inpt, list):
        if all(isinstance(x, int) for x in inpt):
            request.vint.rep_int.extend(inpt)
        elif all(isinstance(x, float) for x in inpt):
            request.vdouble.rep_double.extend(inpt)
    elif isinstance(inpt, field_base._FieldBase):
        request.field.CopyFrom(inpt._message)
    elif isinstance(inpt, collection.Collection):
        request.collection.CopyFrom(inpt._message)
    elif isinstance(inpt, scoping.Scoping):
        request.scoping.CopyFrom(inpt._message)
    elif isinstance(inpt, data_sources.DataSources):
        request.data_sources.CopyFrom(inpt._message)
    elif isinstance(inpt, model.Model):
        request.data_sources.CopyFrom(inpt.metadata.data_sources._message)
    elif isinstance(inpt, meshed_region.MeshedRegion):
        request.mesh.CopyFrom(inpt._message)
    elif isinstance(inpt, cyclic_support.CyclicSupport):
        request.cyc_support.CopyFrom(inpt._message)
    elif isinstance(inpt, Operator):
        request.inputop.inputop.CopyFrom(inpt._message)
        request.inputop.pinOut = pin_out
    elif isinstance(inpt, Output):
        request.inputop.inputop.CopyFrom(inpt._operator._message)
        request.inputop.pinOut = inpt._pin
    else:
        errormsg = f"input type {inpt.__class__} cannot be connected"
        raise TypeError(errormsg)



"""
Custom Fields Containers
========================
Contains the inherited classes from the
:class:`FieldsContainer <ansys.dpf.core.fields_container.FieldsContainer>` class.
These new classes offer helpers to access data for specific usage, such as
results split by body or split by material.
"""
from ansys.dpf.core.fields_container import FieldsContainer
from ansys.grpc.dpf import meshed_region_pb2


class ElShapeFieldsContainer(FieldsContainer):
    """Represents a fields container with fields split by an element shape.

    Instances of this class are created when a model result is split by an
    element shape, such as a solid, shell, or beam.

    Parameters
    ----------
    fields_container : ansys.grpc.dpf.collection_pb2.Collection or FieldsContainer, optional
        Fields container created from either a collection message or by copying an existing
        fields container. The default is ``None``.
    server : server.DPFServer, optional
        Server with the channel connected to the remote or local instance. The default is
        ``None``, in which case an attempt is made to use the global server.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
    >>> fc = model.results.displacement.on_all_time_freqs.split_by_shape.eval()
    >>> len(fc.solid_fields())
    45
    >>> solid_f_time_2 = fc.solid_field(2)

    """

    def __init__(self, fields_container=None, server=None):
        super().__init__(fields_container, server)
        if not fields_container:
            self.add_label("elshape")

    def solid_fields(self, timeid=None, complexid=None):
        """Retrieve a list of all fields with solid element shapes.

        You can filter the list of fields with solid element shapes
        based on a given time, complex type, or both.

        Parameters
        ----------
        timeid : int, optional
            Time ID for filtering fields with solid element shapes.
        complexid : int, optional
            Complex type ID for filtering fields with solid element shapes.
            0 is for real numbers, and 1 is for imaginary numbers.

        Returns
        -------
        list
            List of fields corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
        >>> fc = model.results.displacement.split_by_shape.eval()
        >>> len(fc.solid_fields())
        1
        >>> len(fc.solid_fields(timeid=1))
        1

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        label_space["elshape"] = meshed_region_pb2.ElementShape.Value("SOLID")
        return self.get_fields(label_space)

    def shell_fields(self, timeid=None, complexid=None):
        """Retrieve a list of all fields with shell element shapes.

        You can filter the list of fields with shell element shapes based on
        a given time, complex type, or both.

        Parameters
        ----------
        timeid : int, optional
            Time ID for filtering fields with shell element shapes.
        complexid : int, optional
            Complex type ID for filtering fields with shell element shapes.
            0 is for real numbers, and 1 is for imaginary numbers.

        Returns
        -------
        list
            List of fields corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
        >>> fc = model.results.displacement.on_all_time_freqs.split_by_shape.eval()
        >>> len(fc.shell_fields())
        45
        >>> len(fc.shell_fields(timeid=3))
        1

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        label_space["elshape"] = meshed_region_pb2.ElementShape.Value("SHELL")
        return self.get_fields(label_space)

    def beam_fields(self, timeid=None, complexid=None):
        """Retrieve a list of all fields with beam element shapes.

        You can filter the list of fields with beam element shapes based on
        a given time, complex type, or both.

        Parameters
        ----------
        timeid : int, optional
            Time ID for filtering fields with beam element shapes.
        complexid : int, optional
            Complex type ID for filtering fields with beam element shapes.
            0 is for real numbers, and 1 is for imaginary numbers.

        Returns
        -------
        list
            List of fields corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
        >>> fc = model.results.displacement.on_all_time_freqs.split_by_shape.eval()
        >>> len(fc.beam_fields())
        45
        >>> len(fc.beam_fields(timeid=3))
        1

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        label_space["elshape"] = meshed_region_pb2.ElementShape.Value("BEAM")
        return self.get_fields(label_space)

    def solid_field(self, timeid=None, complexid=None):
        """Retrieve a field with a solid element shape.

        You can give a time, complex type, or both. If the number of fields
        matching the request is higher than one, an exception is raised.

        Parameters
        ----------
        timeid : int, optional
            Time ID for filtering fields with solid element shapes.
        complexid : int, optional
            Complex type ID for filtering fields with solid element shapes.
            0 is for real numbers, and 1 is for imaginary numbers.

        Returns
        -------
        :class:`Field <ansys.dpf.core.field>'
            Field corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
        >>> fc = model.results.displacement.on_all_time_freqs.split_by_shape.eval()
        >>> field = fc.solid_field(timeid=3)

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        label_space["elshape"] = meshed_region_pb2.ElementShape.Value("SOLID")
        return self.get_field(label_space)

    def shell_field(self, timeid=None, complexid=None):
        """Retrieve a field with a shell element shape.

        You can give a time, complex type, or both. If the number of fields
        matching the request is higher than one, an exception is raised.

        Parameters
        ----------
        timeid : int, optional
            Time ID for filtering fields with shell element shapes.
        complexid : int, optional
            Complex type ID for filtering fields with shell element shapes.
            0 is for real numbers, and 1 is for imaginary numbers.


        Returns
        -------
        :class:`Field <ansys.dpf.core.field>'
            Field corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
        >>> fc = model.results.displacement.on_all_time_freqs.split_by_shape.eval()
        >>> field = fc.shell_field(timeid=3)

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        label_space["elshape"] = meshed_region_pb2.ElementShape.Value("SHELL")
        return self.get_field(label_space)

    def beam_field(self, timeid=None, complexid=None):
        """Retrieve a field with a beam element shape.

        You can give a time, complex type, or both. If the number of fields
        matching the request is higher than one, an exception is raised.

        Parameters
        ----------
        timeid : int, optional
            Time ID for filtering fields with solid element shapes.
        complexid : int, optional
            Complex type ID for filtering fields with solid element shapes.
            0 is for real numbers, and 1 is for imaginary numbers.

        Returns
        -------
        :class:`Field <ansys.dpf.core.field>'
            Field corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
        >>> fc = model.results.displacement.on_all_time_freqs.split_by_shape.eval()
        >>> field = fc.beam_field(timeid=3)

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        label_space["elshape"] = meshed_region_pb2.ElementShape.Value("BEAM")
        return self.get_field(label_space)


class BodyFieldsContainer(FieldsContainer):
    """Represents a fields container with fields split by a body.

    Instances of this class are created when a model result is split by a
    body, which is an MAPDL material property.

    Parameters
    ----------
    fields_container : ansys.grpc.dpf.collection_pb2.Collection or FieldsContainer, optional
        Fields container created from either a collection message or by copying an existing
        fields container. The default is ``None``.
    server : server.DPFServer, optional
        Server with the channel connected to the remote or local instance. The default is
        ``None``, in which case an attempt is made to use the
        global server.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
    >>> fc = model.results.displacement.on_all_time_freqs.split_by_body.eval()
    >>> fc.get_mat_scoping().ids[3]
    45
    >>> len(fc.get_fields_by_mat_id(45))
    45
    >>> f_time_2 = fc.get_field_by_mat_id(45, timeid=2)

    """

    def __init__(self, fields_container=None, server=None):
        super().__init__(fields_container, server)
        if not fields_container:
            self.add_label("elshape")

    def get_fields_by_mat_id(self, matid, timeid=None, complexid=None):
        """Retrieve a list of all fields for a material ID.

        You can filter the list of fields for a material ID based on
        a given time, complex type, or both.

        Parameters
        ----------
        matid : int, optional
           Material ID. To request available material IDs, you can use
           the `get_mat_scoping` method.
        timeid : int, optional
            Time ID for filtering fields with the given material ID.
        complexid : int, optional
            Complex type ID for filtering fields with the given material
            ID. 0 is for real numbers, and 1 is for imaginary numbers.

        Returns
        -------
        List
            List of fields corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
        >>> fc = model.results.displacement.on_all_time_freqs.split_by_body.eval()
        >>> len(fc.get_fields_by_mat_id(1))
        45
        >>> len(fc.get_fields_by_mat_id(1, timeid=3))
        1

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        label_space["mat"] = matid
        return self.get_fields(label_space)

    def get_field_by_mat_id(self, matid, timeid=None, complexid=None):
        """Retrieve a field with a given material ID.

         You can filter the field based on a given time, complex type, or both.

        Parameters
        ----------
        matid : int, optional
           Material ID. To request available material IDs, you can use
           the `get_mat_scoping` method.
        timeid : int, optional
            Time ID for filtering fields with the given material ID.
        complexid : int, optional
            Complex type ID for filtering fields with the given material
            ID. 0 is for real numbers, and 1 is for imaginary numbers.

        Returns
        -------
        :class:`Field <ansys.dpf.core.field>'
            Field corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_all_kinds_of_complexity_modal())
        >>> fc = model.results.displacement.on_all_time_freqs.split_by_body.eval()
        >>> f_time_2 = fc.get_field_by_mat_id(45, timeid=2)

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        label_space["mat"] = matid
        return self.get_field(label_space)

    def get_mat_scoping(self):
        """Retrieves the material or body scoping containing material IDs.

        Returns
        -------
        :class:`Scoping <ansys.dpf.core.scoping>'
            Field corresponding to the request.
            Scoping containing the material IDs available in the fields container.
        """
        return self.get_label_scoping("mat")



"""
elements_facets_surfaces_over_time
==================================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "geo" category
"""

class elements_facets_surfaces_over_time(Operator):
    """Calculation of the surface of each element's facet over time of a mesh for each specified time step. Moreover, it gives as output a new mesh made with only surface elements.

      available inputs:
        - scoping (Scoping) (optional)
        - displacement (FieldsContainer) (optional)
        - mesh (MeshedRegion) (optional)

      available outputs:
        - fields_container (FieldsContainer)
        - mesh (MeshedRegion)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()

      >>> # Make input connections
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_displacement = dpf.FieldsContainer()
      >>> op.inputs.displacement.connect(my_displacement)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.geo.elements_facets_surfaces_over_time(scoping=my_scoping,displacement=my_displacement,mesh=my_mesh)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()
      >>> result_mesh = op.outputs.mesh()"""
    def __init__(self, scoping=None, displacement=None, mesh=None, config=None, server=None):
        super().__init__(name="surfaces_provider", config = config, server = server)
        self._inputs = InputsElementsFacetsSurfacesOverTime(self)
        self._outputs = OutputsElementsFacetsSurfacesOverTime(self)
        if scoping !=None:
            self.inputs.scoping.connect(scoping)
        if displacement !=None:
            self.inputs.displacement.connect(displacement)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        spec = Specification(description="""Calculation of the surface of each element's facet over time of a mesh for each specified time step. Moreover, it gives as output a new mesh made with only surface elements.""",
                             map_input_pin_spec={
                                 1 : PinSpecification(name = "scoping", type_names=["scoping"], optional=True, document=""""""), 
                                 2 : PinSpecification(name = "displacement", type_names=["fields_container"], optional=True, document="""Displacement field's container."""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=True, document="""Mesh must be defined if the displacement field's container does not contain it, or if there is no displacement.""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""Surfaces field."""), 
                                 1 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=False, document="""Mesh made of surface elements only.""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "surfaces_provider")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsElementsFacetsSurfacesOverTime 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsElementsFacetsSurfacesOverTime 
        """
        return super().outputs


#internal name: surfaces_provider
#scripting name: elements_facets_surfaces_over_time
class InputsElementsFacetsSurfacesOverTime(_Inputs):
    """Intermediate class used to connect user inputs to elements_facets_surfaces_over_time operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_displacement = dpf.FieldsContainer()
      >>> op.inputs.displacement.connect(my_displacement)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
    """
    def __init__(self, op: Operator):
        super().__init__(elements_facets_surfaces_over_time._spec().inputs, op)
        self._scoping = Input(elements_facets_surfaces_over_time._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._scoping)
        self._displacement = Input(elements_facets_surfaces_over_time._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._displacement)
        self._mesh = Input(elements_facets_surfaces_over_time._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator

        Parameters
        ----------
        my_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> #or
        >>> op.inputs.scoping(my_scoping)

        """
        return self._scoping

    @property
    def displacement(self):
        """Allows to connect displacement input to the operator

        - pindoc: Displacement field's container.

        Parameters
        ----------
        my_displacement : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> op.inputs.displacement.connect(my_displacement)
        >>> #or
        >>> op.inputs.displacement(my_displacement)

        """
        return self._displacement

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: Mesh must be defined if the displacement field's container does not contain it, or if there is no displacement.

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

class OutputsElementsFacetsSurfacesOverTime(_Outputs):
    """Intermediate class used to get outputs from elements_facets_surfaces_over_time operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
      >>> result_mesh = op.outputs.mesh()
    """
    def __init__(self, op: Operator):
        super().__init__(elements_facets_surfaces_over_time._spec().outputs, op)
        self._fields_container = Output(elements_facets_surfaces_over_time._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)
        self._mesh = Output(elements_facets_surfaces_over_time._spec().output_pin(1), 1, op) 
        self._outputs.append(self._mesh)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        - pindoc: Surfaces field.

        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container

    @property
    def mesh(self):
        """Allows to get mesh output of the operator


        - pindoc: Mesh made of surface elements only.

        Returns
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh = op.outputs.mesh() 
        """
        return self._mesh

